c
c  program slipmake
c
c $Id: slipmake.f,v 1.16 2010/04/02 18:03:27 agnew Exp agnew $
c 
c $Log: slipmake.f,v $
c Revision 1.16  2010/04/02 18:03:27  agnew
c further corrections to depth handling: all depths input and output as
c positive down
c
c Revision 1.15  2010/04/02 16:49:39  agnew
c fixed signs of depth and corner depths as written out
c
c Revision 1.14  2010/03/27 18:06:43  agnew
c corrected handling of width command, made this default if neither width
c nor depth specified, modified all depths to be negative inside program
c
c Revision 1.13  2010/03/19 22:39:56  agnew
c revised fault projection to use geodesics instead of XYZ, to remove
c problem with projecting low-angle faults. Also added  width  command
c to allow subdivision in width.
c
c Revision 1.12  2009/09/17 02:03:06  agnew
c output centroid in all cases
c
c Revision 1.11  2009/09/14 15:24:31  agnew
c added listing of area to output
c
c Revision 1.10  2009/09/03 23:25:02  agnew
c Modified handling of start and end times, and changed commands (see icheck)
c
c Revision 1.9  2009/09/03 05:08:55  agnew
c Fixed lntime function in cumslp. Added centroid coordinates/distance output
c
c Revision 1.8  2009/09/02 16:43:42  agnew
c Added centroid command, checking of inputs, and defaults for depth
c and length
c
c Revision 1.7  2009/06/12 22:04:18  agnew
c fixed slip output format
c
c Revision 1.6  2009/06/11 20:52:53  agnew
c modified input format, added linear slip, corrected handling of depth
c
c Revision 1.5  2009/06/08 22:29:47  agnew
c modified fault-projection iteration
c
c Revision 1.4  2009/06/03 00:49:43  agnew
c Added ability to project and subdivide fault planes
c
c Revision 1.3  2009/05/27 21:42:38  agnew
c Removed subroutines in common with fakenet (now in fakesubs.f)
c
c Revision 1.2  2009/05/27 00:56:55  agnew
c Tested equalization and propagation, fixed initial bugs,
c modified to be meshable with fakenet
c
c Revision 1.1  2009/05/26 20:44:21  agnew
c Initial revision
c
c
c  Reads a file of fault planes, and creates a time-history of slip on them,
c    for use in fakenet
c
      parameter (inmx=500)
      parameter (maxpl=2000)
      double precision begslp,endslp
      double precision begin, end, sample
      character*1 lcode,dcode
c
c common block for overall start and end
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 nflt
c
c common block for fault-plane corner locations and centroids
c
      common/fltpos/fclat(maxpl,4),fclong(maxpl,4),fcd(maxpl,4),
     1              cflat(maxpl),cflong(maxpl),area(maxpl)
c
c common block for fault slip parameterization
c
      common/slipar/begslp,endslp,fs(maxpl),params(3),propa(2),indf
c
c common blocks of scanrp for handling commands:
c
      character*80 input(inmx)
      common /store/ input
      common /ndict/ iecho,nin
c
c Read commands; completed when reads  exec  command
c
      call scanrp
c
c Read fault slip information
c
      call rdflt
c
c Find the fault planes, subsegmented to have specified depths and lengths
c
      call fltproj
c
c Read slip-history specification
c
      call rdslip
c
c Compute and write out the results
c
      call slipmake
c
      stop
      end
c=======================================================================
      subroutine rdflt
c
c$$$$ calls getarr,getchr,getone
c
c  Reads in fault plane and slip information
c
      parameter (maxpl=2000)
      character*80 name,line
      character*1 lcode,dcode
      double precision begslp,endslp
      double precision begin, end, sample
      dimension it(5),rit(5),itm(6)
c
c common block for overall start and end
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for fault slip parameterization
c
      common/slipar/begslp,endslp,fs(maxpl),params(3),propa(2),indf
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 nflt
      data lui/11/
      data nflt/0/
      data sample/1./
c
c get sample interval
c
      call getone('samp',tmp,iok)
      if(iok.eq.0) then
        write(6,100)
 100    format(' Command  sample  needs interval in days ')
        stop
      endif
      sample=tmp
c
c get temporal limits--which are converted to Julian date
c
      call getarr('star',rit,5,nt)
      if(nt.lt.0) then
        write(6,102)
 102    format(' Need start time of slip ')
        stop
      endif
      if(nt.lt.2) then
        write(6,103)
 103    format(' Command  startslip  needs time in y d [h m s] ')
        stop
      endif
      do i=1,5
        it(i)=0
      enddo
      do i=1,nt
        it(i)=rit(i)
      enddo
      call toymd(it,itm)
      begslp=dat2jul(itm)
      call getarr('end ',rit,5,nt)
      if(nt.ge.0.and.nt.lt.2) then
        write(6,104)
 104    format(' Command  end  needs time in y d [h m s] ')
        stop
      endif
      do i=1,5
        it(i)=0
      enddo
      if(nt.ge.2) then
        do i=1,nt
          it(i)=rit(i)
        enddo
      endif
c
c  if no end command, make time very late
c
      if(nt.lt.0) then
        it(1)=2109
        it(2)=1
      endif
      call toymd(it,itm)
      endslp=dat2jul(itm)
      if(begslp.ge.endslp) then
        write(6,106)
 106    format('Begin time of slip must precede end time')
        stop
      endif
c
c get input filename, and read the data
c
      call getchr('faul',name,iok)
      if (iok.le.0) then
        write(6,108)
 108    format(' Need name of input (fault geometry) file')
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,110,end=7) line(1:80)
 110  format(a80)
c
c data read in as
c  
c lat1 long1 lat2 long2 depth width dip
c
c where depth is of the top of the fault plane, and is reckoned with positive
c down, with zero being on the ellipsoid
c
      nflt=nflt+1
      read(line,*) gt1(nflt),gg1(nflt),gt2(nflt),gg2(nflt),
     1  dep(nflt),wid(nflt),dip(nflt)
c
c set depths to meters, positive up; depths above -200 are set to zero,
c  to squash the topography
c
      dep(nflt)=-1000.*dep(nflt)
      wid(nflt)=1000.*wid(nflt)
      if(dep(nflt).gt.-200) dep(nflt)=0
      go to 5
 7    continue
      close(lui)
      return
      end
c=======================================================================
      subroutine fltproj
      parameter (maxpl=2000)
      double precision sxyz,cxyz,rnu,rho
      character*1 lcode,dcode
      character*3 centyp
      dimension dipt(maxpl),dept(maxpl),ett(maxpl),egt(maxpl),
     1  stft(maxpl),dist(maxpl),widt(maxpl)
      dimension cxyz(3,4),sxyz(3),w(2),depth(3)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 nflt
c
c common block for fault-plane corner locations and centroids
c
      common/fltpos/fclat(maxpl,4),fclong(maxpl,4),fcd(maxpl,4),
     1              cflat(maxpl),cflong(maxpl),area(maxpl)
c
      data dr/.0174533/
c
c get subsegment parameters: length is along-strike. width or depth are
c down-dip or vertical (one or the other)
c
      call getone('leng',seglen,jleng)
      if(jleng.eq.0) then
        write(6,100)
 100    format(' Command  length  needs segment length in km ')
        stop
      endif
      if(jleng.ge.1) seglen=1000.*seglen
c
c  Now get the depth or width parameters (not both)
c
      call getarr('dept',depth,3,jdep)
      call getone('widt',segwid,jwid)
      if(jdep.eq.0) then
        write(6,102)
 102    format(' Command  depth  needs  top [bottom [[interval]] ')
        stop
      endif
      if(jwid.eq.0) then
        write(6,104)
 104    format(' Command  width  needs number of subsegments')
        stop
      endif
      if(jwid.gt.0.and.jdep.gt.0) then
        write(6,106)
 106    format(' Cannot specify both  width  and  depth')
        stop
      endif
c
c  Set jwid so that >0 is width, 0 is depth, <0 is neither
c
      if(jdep.gt.0) jwid=0
c
c depth may have 1-3 parameters; if 3, set number of intervals using these;
c   otherwise only one interval; if bottom depth not specified (1 parameter)
c   use bottom of fault plane. Spacing in depth is set only if there are
c   three parameters.
c depths read in as positive numbers, and changed to negative
c
      ndepth=1
      if(jdep.gt.0) then
        do i=1,jdep
         depth(i)=-1000.*abs(depth(i))
        enddo
        if(jdep.eq.3) ndepth=max(nint((depth(2)-depth(1))/depth(3)),1)
        if(jdep.ge.2) ddep=(depth(2)-depth(1))/ndepth
      endif
c
c if we have a segment width, convert it to meters
c
      if(jwid.gt.0) segwid=1000.*segwid
c
c  get centroid specification
c
      call getchr('cent',centyp,jcen)
      if(jcen.le.0) centyp='top'
      if(centyp.ne.'top'.and.centyp.ne.'mid') then
        write(6,108)
 108    format(' Command  centroid  needs argument of  top  or  mid')
        stop
      endif
c
c convert positions of fault ends to strike (forward azimuth at location
c saved), and length; store widths, dips, and depths
c
      do i=1,nflt
        call tostr(gt1(i),gg1(i),gt2(i),gg2(i),stft(i),strb,dist(i))
        ett(i) = gt1(i)
        egt(i) = gg1(i)
        dipt(i) = dip(i)
        widt(i) = wid(i)
        dept(i) = dep(i)
      enddo
      nt = nflt
c
c we now have moved all the fault data from the common block, which will be
c  overwritten
c
c Divide length into subsegments. Then, for each subsegment:
c
c    1. Find the center point, and at that point find the local radius of
c       curvature in the direction of dip.
c
c    2. If the fault is being divided by depth, find the horizontal distance
c       on the surface to the upper and lower edges, assuming the fault has
c       a constant dip with depth. If the fault is being divided by width,
c       do the same by dividing the downdip width into equal subsegments.
c
c    3. Determine the coordinates of these corners by projecting at right
c       angles to the top of the fault, by the appropriate distances.
c
c    4. Convert the coordinates from lat/long/ht to XYZ and find the fault
c       width from these (this makes the actual fault surface between the
c       corners a plane).
c
c    5. Use XYZ to find centroid, convert this to lat/long/ht 
c 
c  start of loop over input faults
c
      nflt = 1
      do kk=1,nt
        d=dist(kk)
        if(jleng.eq.1) nseg=max(nint(d/seglen),1)
        if(jleng.lt.1) nseg=1
        ds=d/nseg

c no depth command, set depth to top of fault
c
c If we have set no depths, the top depth is the top of the fault;
c if we have set only the top depth, find the bottom depth of
c  the fault; either way, set the depth interval to be the full interval
c  note that depths (and interval) are all negative
c
        if(jdep.lt.0) depth(1)=dept(kk)
        if(jdep.eq.1) then
          depth(2)=dept(kk)-widt(kk)*sin(dr*dipt(kk))
          ddep=depth(2)-depth(1)
        endif
c
c find dip parameter. If this is close to zero (no dip) or infinity (vertical
c  fault) set it to values that are close to these but that will avoid
c  divisions by zero
        sdp=sin(dr*dipt(kk))
        cdp=cos(dr*dipt(kk))
        if(abs(cdp).lt.1.e-4.and.cdp.ge.0) cdp=1.e-4
        if(abs(cdp).lt.1.e-4.and.cdp.lt.0) cdp=-1.e-4
        if(abs(sdp).lt.1.e-4.and.sdp.ge.0) sdp=1.e-4
        if(abs(sdp).lt.1.e-4.and.sdp.lt.0) sdp=-1.e-4
        b=sdp/cdp
c
c set number of steps in width (use ndepth for this also)
c
        if(jwid.lt.0) ndepth=1
        if(jwid.gt.0) ndepth=max(nint(widt(kk)/segwid),1)
c
c if subdividing by fault width, find horizontal and vertical steps that
c  correspond to width subdivision; set ndepth to number of subdivisions
c
        if(jwid.ne.0) then
          ddep = -(widt(kk)/ndepth)*sdp
          dh = (widt(kk)/ndepth)*cdp
        endif
c 
c  start of loop over subsegments in length: find start and stop of segment
c  on geodesic between two points
c
        do i=1,nseg
          dn=ds*(i-1)
          df=ds*i
          dm=(dn+df)/2
          call frmstr(ett(kk),egt(kk),stft(kk),dn,tnt,tng)
          call frmstr(ett(kk),egt(kk),stft(kk),df,tft,tfg)
          call frmstr(ett(kk),egt(kk),stft(kk),dm,tmt,tmg)
c tmt and tmg are coordinates of center point of segment; find strike of
c this subsegment, and local radius at center in direction perpendicular
c to this.
          call rads(tmt,rnu,rho)
          call tostr(tnt,tng,tft,tfg,sf,sb,dj)
          sb=sb-180
          dipd = (sf+sb)/2 + 90.
          rho = cos(dr*dipd)**2/rho**2 + sin(dr*dipd)**2/rnu**2
          rho = sqrt(1./rho) - dept(kk)
c
c  Now ready to project fault. If depth command is set, project fault
c   plane, for each subsegment in length, up or down dip to get the
c   depth limits asked for. If width command is set, subdivide the
c   fault plane into subsegments down-dip. If dip is zero, only the
c   second command may be used.
c  Issue warning if depth is above top of fault
c
          do ii=1,ndepth
            dpt=depth(1)+(ii-1)*ddep
            dpb=depth(1)+ii*ddep
            if(dpt.gt.dept(kk)) then
              write(6,110) dpt/1000,dept(kk)/1000
 110          format(' ***Top depth of ',f6.1,' is above top of fault',
     1          ' plane at ',f6.1)
            endif
            dpt=dpt-dept(kk)
            dpb=dpb-dept(kk)
c
c  Unless subdividing by depth, use the depths relative to the top of the plane
c  to find the horizontal distances for both depths; these are used for lines
c  from both ends. Depths of corners are reckoned as negative since they are
c  later used in getting XYZ positions.
c
            if(jwid.eq.0) dis=(-rho/b)*log((rho+dpt)/rho)
            if(jwid.ne.0) dis=(ii-1)*dh
            call frmstr(tnt,tng,dipd,dis,fclat(nflt,1), 
     1                                  fclong(nflt,1))
            fcd(nflt,1) = (dpt + dept(kk))
            dep(nflt) = fcd(nflt,1)/1000
            gt1(nflt) = fclat(nflt,1)
            gg1(nflt) = fclong(nflt,1)
c
c bottom corners
c
            if(jwid.eq.0) dis=(-rho/b)*log((rho+dpb)/rho)
            if(jwid.ne.0) dis=ii*dh
            call frmstr(tnt,tng,dipd,dis,fclat(nflt,2), 
     1                                  fclong(nflt,2))
            fcd(nflt,2) = (dpb + dept(kk))
            call frmstr(tft,tfg,dipd,dis,fclat(nflt,3), 
     1                                  fclong(nflt,3))
            fcd(nflt,3) = (dpb + dept(kk))
c
c top corner
c
            if(jwid.eq.0) dis=(-rho/b)*log((rho+dpt)/rho)
            if(jwid.ne.0) dis=(ii-1)*dh
            call frmstr(tft,tfg,dipd,dis,fclat(nflt,4), 
     1                                  fclong(nflt,4))
            fcd(nflt,4) = (dpt + dept(kk))
            gt2(nflt) = fclat(nflt,4)
            gg2(nflt) = fclong(nflt,4)
c 
c  find the XYZ values for the corners
c
            do j=1,4
              call toxyz(fclat(nflt,j),fclong(nflt,j),fcd(nflt,j),
     1        sxyz)
              do k=1,3
                cxyz(k,j)=sxyz(k)
              enddo
            enddo
c
c  find centroid; default is top (to match original useage)
c
            do k=1,3
              sxyz(k)=0
              if(centyp.eq.'top') then
                sxyz(k)=sxyz(k)+cxyz(k,1)/2+cxyz(k,4)/2
              endif
              if(centyp.eq.'mid') then
                do j=1,4
                  sxyz(k)=sxyz(k)+cxyz(k,j)/4
                enddo
              endif
            enddo
            call frmxyz(sxyz,cflat(nflt),cflong(nflt),rd1)
c
c  find width (scaled back to km)
c
            w(1)=0
            w(2)=0
            do k=1,3
              w(1) = w(1) + (cxyz(k,1)-cxyz(k,2))**2
              w(2) = w(2) + (cxyz(k,4)-cxyz(k,3))**2
            enddo
            wid(nflt)=(sqrt(w(1))+sqrt(w(2)))/2000
            dip(nflt) = dipt(kk)
            call tostr(gt1(nflt),gg1(nflt),gt2(nflt),gg2(nflt),sf,sb,d)
            area(nflt)=(d/1000)*wid(nflt)
            lcode(nflt) = "t"
            dcode(nflt) = "t"
            nflt=nflt+1
c
c end of loop over depths
c
          enddo
c
c end of loop over subsegments
c
        enddo
c
c end of loop over faults
c
      enddo
      nflt=nflt-1
      return
      end
c=======================================================================
      subroutine rdslip
      parameter (maxpl=2000)
      parameter (maxfu=6)
      double precision begslp,endslp
      double precision begin, end, sample
      character*1 lcode,dcode
      character*1 spred
      character*4 fname
      character*20 name
      dimension fname(maxfu),npar(maxfu)
c
c common block for overall start and end
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 nflt
c
c common block for fault-plane corner locations and centroids
c
      common/fltpos/fclat(maxpl,4),fclong(maxpl,4),fcd(maxpl,4),
     1              cflat(maxpl),cflong(maxpl),area(maxpl)
c
c common block for fault slip parameterization
c
      common/slipar/begslp,endslp,fs(maxpl),params(3),propa(2),indf
c
      data spred/'s'/
      data nfunc/maxfu/
      data fname/'logl','burr','weib','line','lnti','cosi'/
      data npar/1,2,1,0,1,1/
c
c get rate model, parameters, and propagation rate (if any)
c
      call getchr('rate',name,iok)
      indf=0
      do i=1,nfunc
        if(name(1:4).eq.fname(i)) indf=i
      enddo
      if(indf.eq.0) then
        write(6,100) (fname(i),i=1,nfunc)
 100    format(' Command  ratefunc  needs as argument one of: ',/,
     1    20x,4(a4,', '),' or ',a4)
        stop
      endif
      call getarr('para',params,3,iok)
      if(npar(indf).gt.iok.and.npar(indf).gt.0) then
        write(6,102) fname(indf),npar(indf)
 102    format(' Time function ',a4,' needs to have ',i2,' arguments')
        stop
      endif
      call getarr('prop',propa,2,iok)
      if(iok.ge.0.and.iok.lt.2) then
        write(6,104)
 104    format(' Command  propagate needs speed and direction')
        stop
      endif
      if(iok.lt.0) propa(1)=-1
c
c get total moment--if negative, assume it is moment magnitude
c
      call getone('mome',tmom,iok)
      if (iok.le.0) then
        write(6,106)
 106    format(' Need total moment (N-m, Mw if <0) or mm/yr for linear')
        stop
      endif
      if(tmom.lt.0) tmom=10**(1.5*(-tmom+6))
      ta=0
      do i=1,nflt
        call tostr(gt1(i),gg1(i),gt2(i),gg2(i),fa,ba,d)
        area(i)=(d/1000)*wid(i)
        ta=ta+area(i)
      enddo
c
c convert to slip on each fault plane (either equal slip, or equal moment)
c
      if(indf.ne.4) then
        call getchr('equa',spred,iok)
        if(iok.eq.0.or.(spred.ne.'s'.and.spred.ne.'m')) then
          write(6,108)
 108      format(' Command  equalize  needs argument of  s  or  m')
          stop
        endif
c
c convert moment to average slip assuming moment in N-m, area in km**2;
c  then store final slip on each segment
c
        ts=tmom/(3.e16*ta)
        do i=1,nflt
          fs(i)=area(i)/ta
        enddo
        do i=1,nflt
          if(spred.eq.'s') fs(i)=ts
          if(spred.eq.'m') fs(i)=ts/(nflt*fs(i))
        enddo
      endif
c
c if the slip model is a linear trend, the input value will be taken
c   to be the slip rate in mm/yr
c
      if(indf.eq.4) then
        do i=1,nflt
          fs(i)=abs(tmom)/1000.
        enddo
      endif
      return
      end
c=======================================================================
      subroutine slipmake
      parameter (maxpl=2000)
      double precision time
      double precision begslp,endslp
      double precision begin, end, sample
      character*1 lcode,dcode
      character*80 name
      dimension delay(maxpl),ddist(maxpl)
      dimension it(5),itm(6)
c
c common block for overall start and end
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 nflt
c
c common block for fault-plane corner locations and centroids
c
      common/fltpos/fclat(maxpl,4),fclong(maxpl,4),fcd(maxpl,4),
     1              cflat(maxpl),cflong(maxpl),area(maxpl)
c
c common block for fault slip parameterization
c
      common/slipar/begslp,endslp,fs(maxpl),params(3),propa(2),indf
c
      data luo/12/
      data dr/.0174533/
      data delay/maxpl*0./
c
c find number of epochs
c
      nep=nint((endslp-begslp)/sample+1)
c
c get the output file name
c
      call getchr('outp',name,iok)
      if (iok.le.0) then
        write(6,100)
 100    format(' Need name of input (fault slip) file')
        stop
      endif
      open(luo,file=name)
c
c get the rake
c
      call getone('rake',rake,iok)
      if (iok.le.0) then
        write(6,102)
 102    format(' Need rake of slip ')
        stop
      endif
c
c If propagating, find time delay between segments by finding the vector
c   direction from the centroid of the fault centers to each fault center,
c   and then taking the dot product of that with the unit vector along
c   the chosen azimuth, and through the centroid
c
      call centroid(cflat,cflong,nflt,cenlat,cenlong)
      if(propa(1).gt.0) then
        do i=1,nflt
           call tostr(cenlat,cenlong,cflat(i),cflong(i),fa,ba,d)
           ddist(i)=(d/1000)*cos(dr*(fa-propa(2)))
           delay(i)=ddist(i)/propa(1)
           if(i.eq.1) dmin=delay(i)
           if(i.gt.1) dmin=amin1(dmin,delay(i))
        enddo
      endif
c
c set minimum delay to zero
c
      do i=1,nflt
         delay(i)=delay(i)-dmin
      enddo
c
c Now, for each segment, compute the time history, and output the segment
c   information, the segment delay and corners of the segment (neither
c    used by fakenet) and the slip history
c Note that output depths are converted to positive down
c
      do i=1,nflt
       write(luo,110) gt1(i),gg1(i),gt2(i),gg2(i),
     1     lcode(i),dcode(i),-dep(i),wid(i),dip(i)
 110   format('g',2(f12.4,f10.4),1x,a1,1x,a1,3f6.1)
       if(propa(1).gt.0) then
         write(luo,112) cflat(i),cflong(i),delay(i),ddist(i)
 112     format('d',f10.4,f12.4,1x,f10.5,f8.1)
       endif
       if(propa(1).le.0) then
         write(luo,113) cflat(i),cflong(i)
 113     format('d',f10.4,f12.4)
       endif
       do k=1,4
         write(luo,114) fclat(i,k),fclong(i,k),-fcd(i,k)/1000.
 114     format('c',f10.4,f12.4,f5.1)
       enddo
       write(luo,116) area(i)
 116   format('a',1x,f14.6)
       oslp=0
c
c loop over all epochs; time is normalized by time constant (or to years,
c  for linear trend case)
c
       do j=1,nep
         time=begslp+(j-1)*sample
         call jul2dat(time,itm)
         call toyd(itm,it)
         if(indf.ne.4) tn=((j-1)*sample-delay(i))/params(1)
         if(indf.eq.4) tn=((j-1)*sample-delay(i))/365.25
         if(tn.le.0) slp=0
         if(tn.gt.0) slp=fs(i)*cumslp(indf,tn,params(2))
         dslp=slp-oslp
c
c write out cumulative slip only if 0.1 mm or more
c
         if(abs(dslp).gt.0.00010) then
            write(luo,120) dslp,nint(rake),it
 120        format('s',f12.5,i5,i6,i4,3i3)
            oslp=slp
         endif
       enddo
      enddo
      return
      end
c=======================================================================
      subroutine tostr(t1,g1,t2,g2,stf,stb,dis)
c
c given a pair of coordinates, returns the two azimuths and the distance
c   in meters
c
      double precision glat1,glon1,glat2,glon2,faz,baz,s
      glat1 = t1
      glon1 = g1
      glat2 = t2
      glon2 = g2
      call gdesinv(glat1,glon1,glat2,glon2,faz,baz,s)
      stf=faz
      stb=baz
      dis=s
      return
      end
c =======================================================================
      subroutine frmstr(t1,g1,str,dis,t2,g2)
c
c given coordinates and an azimuth and distance, returns the coordinates
c   at the end of the geodesic
c
      double precision glat1,glon1,glat2,glon2,faz,baz,s
      glat1 = t1
      glon1 = g1
      faz = str
      s = dis
      call gdesfor(glat1,glon1,glat2,glon2,faz,baz,s)
      t2 = glat2
      g2 = glon2
      return
      end
c =======================================================================
      subroutine gdesinv(glat1,glon1,glat2,glon2,faz,baz,s)
c
c  For specified coordinates, returns the forward and back azimuths
c   and the geodesic distance
c
c Solves the geodetic inverse problem after T. Vincenty:
c modified Rainsford's method with Helmert's elliptical terms.
c Effective in any azimuth and at any distance short of antipodal;
c standpoint/forepoint must not be the geographic pole
c
c Latitudes and longitudes in degrees positive north and east
c Azimuths in degrees from north
c Distance in meters
c
c Programmed by LtCdr L. Pfeifer NGS 18 Feb 1975; modified by
c John G Gergen NGS July 1975
c WGS-84 constants put in, angles changed to degrees by D. C. Agnew Mar 2010
c
      implicit real*8 (a-h,o-z)
      data pi/3.1415926535897932384626d0/
      data a/6378137.d0/
      data fl/298.257222101d0/
      data eps/0.5d-13/
      dr = pi/180.d0
      rd = 180.d0/pi
      f = 1.d0/fl
      r = 1.d0 - f
      tu1 = r*dsin(dr*glat1)/dcos(dr*glat1)
      tu2 = r*dsin(dr*glat2)/dcos(dr*glat2)
      cu1 = 1./dsqrt(tu1*tu1 + 1.)
      su1 = cu1*tu1
      cu2 = 1./dsqrt(tu2*tu2 + 1.)
      s = cu1*cu2
      baz = s*tu2
      faz = baz*tu1
      x = dr*(glon2 - glon1)
c
c loop starts here
c
 3    sx = dsin(x)
      cx = dcos(x)
      tu1 = cu2*sx
      tu2 = baz - su1*cu2*cx
      sy = dsqrt(tu1*tu1 + tu2*tu2)
      cy = s*cx + faz
      y = datan2(sy,cy)
      sa = s*sx/sy
      c2a = -sa*sa + 1.
      cz = faz + faz
      if(c2a.gt.0.)cz = -cz/c2a + cy
      e = cz*cz*2. - 1.
      c = ((-3.*c2a + 4.)*f + 4.)*c2a*f/16.
      d = x
      x = ((e*cy*c + cz)*sy*c + y)*sa
      x = (1. - c)*x*f + dr*(glon2 - glon1)
      if(dabs(d-x).gt.eps) go to 3
c
c done iterating
c
      faz = rd*datan2(tu1,tu2)
      baz = rd*datan2(cu1*sx,baz*cx - su1*cu2) + 180.d0
      x = dsqrt((1./r/r - 1.)*c2a + 1.) + 1.
      x = (x - 2.)/x
      c = 1. - x
      c = (x*x/4. + 1.)/c
      d = (0.375*x*x - 1.)*x
      x = e*cy
      s = 1. - e - e
      s = ((((sy*sy*4. - 3.)*s*cz*d/6. - x)*d/4. + cz)*sy*d + y)*c*a*r
      return
      end
c =======================================================================
      subroutine gdesfor(glat1,glon1,glat2,glon2,faz,baz,s)
c
c  For specified coordinates, forward azimuth, and distance, returns
c   the coordinates
c
c Solves the geodetic forward problem after T. Vincenty:
c modified Rainsford's method with Helmert's elliptical terms.
c Effective in any azimuth and at any distance short of antipodal.
c
c Latitudes and longitudes in degrees positive north and east
c Azimuths in degrees from north
c Distance in meters 
c
c Programmed by LtCdr L. Pfeifer NGS 18 Feb 1975; modified by
c John G Gergen NGS July 1975
c WGS-84 constants put in, angles changed to degrees by D. C. Agnew Mar 2010
c
      implicit real*8 (a-h,o-z)
      data pi/3.1415926535897932384626d0/
      data a/6378137.d0/
      data fl/298.257222101d0/
      data eps/0.5d-13/
      dr = pi/180.d0
      rd = 180.d0/pi
      f = 1.d0/fl
      r = 1. - f
      tu = r*dsin(dr*glat1)/dcos(dr*glat1)
      sf = dsin(dr*faz)
      cf = dcos(dr*faz)
      baz = 0.
      if(cf.ne.0.) baz = datan2(tu,cf)*2.
      cu = 1./dsqrt(tu*tu + 1.)
      su = tu*cu
      sa = cu*sf
      c2a = -sa*sa + 1.
      x = dsqrt((1./r/r - 1.)*c2a + 1.) + 1.
      x = (x - 2.)/x
      c = 1. - x
      c = (x*x/4. + 1)/c
      d = (0.375d0*x*x - 1.)*x
      tu = s/r/a/c
      y = tu
c
c loop starts here
c
 3    sy = dsin(y)
      cy = dcos(y)
      cz = dcos(baz + y)
      e = cz*cz*2. - 1.
      c = y
      x = e*cy
      y = e + e - 1.
      y = (((sy*sy*4. - 3.)*y*cz*d/6. + x)*d/4. - cz)*sy*d + tu
      if(dabs(y-c).gt.eps) go to 3
c
c done iterating
c
      baz = cu*cy*cf - su*sy
      c = r*dsqrt(sa*sa + baz*baz)
      d = su*cy + cu*sy*cf
      glat2 = rd*datan2(d,c)
      c = cu*cy - su*sy*cf
      x = datan2(sy*sf,c)
      c = ((-3.*c2a + 4.)*f + 4.)*c2a*f/16.
      d = ((e*cy*c + cz)*sy*c + y)*sa
      glon2 = glon1  +  rd*(x - (1. - c)*d*f)
      baz = rd*datan2(sa,baz) + 180.d0
      return
      end
c=======================================================================
      subroutine rads(rlat,rnu,rho)
c
c     Given latitude, find the local radii of curvature rnu is NS, rho is EW
c
      double precision s,f,dr,e2,rnu,rho,ae,pi
      data ae/6378137.d0/,finv/298.257223563/,pi/3.1415926536d0/
      dr = pi/180.d0
      f  = 1./finv
      e2 = 2.*f - f*f
      s = sin(dr*rlat)
      rnu = ae/(sqrt(1.-e2*s*s))
      rho = ae*(1-e2)/((1.-e2*s*s)**1.5)
      return
      end
c=======================================================================
      function cumslp(itype,x,p)
c
c Computes a variety of cumulative distribution functions, using these as
c  possible descriptions of cumulative moment release. The input variables are
c 
c    itype - function to be used
c            1 is log-logistic                        1 parameter
c            2 is generalized log-logistic            2
c            3 is Weibull                             1
c            4 is linear trend                        0
c            5 is log                                 0
c            6 is half-cycle of a cosine              0
c    x     - input argument, assumed to be > 0 and normalized by the scaling
c              parameter
c    p     - shape parameter(s)
c
      dimension p(*)
      data pi/3.14159265/
      cumslp=0
      if(x.le.0) return
      if(itype.eq.1) then
        cumslp = (x**p(1))/(1+x**p(1))
      endif
      if(itype.eq.2) then
        cumslp = 1 - ((1+x**p(1))**-p(2))
      endif
      if(itype.eq.3) then
        cumslp = 1 - exp(-x**p(1))
      endif
      if(itype.eq.4) then
        cumslp = x
      endif
      if(itype.eq.5) then
        cumslp = log(1+x)
      endif
      if(itype.eq.6) then
        if(x.le.1) cumslp = sin(pi*x)
      endif
      return
      end
c=======================================================================
c ***********************************************************************
c *                                                                     *
c *   Routines below for reading and processing script commands are     *
c *     provided courtesy of R. L. Parker; specific commands in review  *
c *     listing in scanrp, and checking in icheck, have been modified   *
c *     for this code.                                                  *
c *                                                                     *
c ***********************************************************************
c=======================================================================
      subroutine scanrp
c
c$$$$ calls clear getchr icheck
c
c  Input routine for commands 
c
c  Reads from the standard input until eof or code "execute " or "quit".
c  Saves lines in the input store  /store/ for later retrieval by
c  getarr, getone or getchr.
c
c  Prints a glossary upon request
c
      parameter (inmx=500)
      character*4 code
      character*80 input(inmx),line
      common /store/ input
      common /ndict/ iecho,nin
c
      if (nin .eq. 0) write(6,'(a)') ' ',
     $'Enter commands for producing fault slip file (? for help)'
c
      do 1500 l=nin+1, inmx
        read(*,'(80a)', end=3000) line
        if (line(1:4).eq.'cont' .or. line(1:4).eq.'exec') goto 2000
c  List a glossary of codes
        if (line(1:1) .eq. '?') then
          write(6, '(a/a/(2x,a))')
     $'Enter commands from the following list:',
     $'?:        Remind me of the command list again',
     $'execute:           Quit reading commands and begin calculations',
     $'faults filename:          Enter filename for faults [and slips]',
     $'output filename:                 Enter filename for output file',
     $'startslip yr day [h m s]:          Set start time of slip pulse',
     $'end yr day [h m s]:                      Set end time of output',
     $'sample days:                        Set sample interval in days',
     $'moment tmom:                           Set total moment release',
     $'rake angle:                  Set direction of slip (180 for rl)',
     $'equalize m|s:           Set moment distribution across segments',
     $'depth t b i: Set top, bottom, interval of depths of subsections',
     $'width w:                    Set width (down-dip) of subsections',
     $'length l:              Set length (along-strike) of subsections',
     $'centroid top|mid:         Centroid to be taken at top or middle',
     $'ratecurve name:                   Set type of moment-rate curve',
     $'parameters width [shape1[shape2]]:  Set parameters of m-r curve',
     $'propagate speed azimuth:    Set propagation speed and dir (opt)'
          write(6, '(2x,a)')
     $'review:                           Display current command stack',
     $'clear command:     Delete most recent occurrence of the command',
     $' '
c
c  Review the command stack
        elseif (line(1:4) .eq. 'revi') then
          write(6,'(5x,a)')' ', '=================== ',
     $    (input(i)(1:60),i=1,nin),'=================== '
c
        elseif (line(1:1) .ne. ' ') then
          if (line(1:4) .eq. 'echo') iecho=-iecho
          if (line(1:4) .eq. 'quit') then
            write(6,'(/a)') 'Normal termination'
            stop
          endif
          nin=nin + 1
          if (nin .gt. inmx) then
            write(6,'(a)') '>>>> Too many commands - memory full'
            stop
          endif
          call icheck(line)
          input(nin)=line
c  Clear a command and clear the clear command
          if (line(1:4) .eq. 'clea') then
            call getchr('clear', code, klear)
            call clear (code)
            nin=nin - 1
          endif
        endif
 1500 continue
c
 2000 continue
      n1=max(1, nin-24)
      write(6,'(5x,a)')' ', '=================== ',
     $(input(i)(1:60),i=n1,nin),'=================== '
      return
c
 3000 stop
      end
c=======================================================================
      subroutine icheck(line)
c
c$$$$ calls no other routines
c
c  Checks the command fragment com against a list; appends a
c  warning if com is not present in the list.
      character*80 line
      character*4 com
c
c   lines below are to handle changes in commands made in later versions,
c  to provide backwards compatibility
c
      if(line(1:4).eq.'begs') line(1:4)='star'
      if(line(1:4).eq.'ends') line(1:4)='end '
      com=line(1:4)
      if (0.eq.
     1 index('star end  samp outp faul prop mome rate para',com)+
     2 index('equa rake dept leng cent widt',com))
     2 line=line(1:20)//' <<<<<<< Unrecognized command'
      return
      end
c=======================================================================
